/*
 * $URL$
 * $Id$
 * $Revision$
 */

package org.postgresql.test.jdbc2;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import junit.framework.TestCase;
import org.postgresql.test.TestUtil;

/** Test reading form bytea streams.
 *
 * @author Radosław Smogura
 */
public class BinaryStreamTest extends TestCase {
    private byte[] test_bytes;
    private Connection con;
    protected void setUp() throws Exception {
        prepareTestBytesFromDriverClass();
        
        con = TestUtil.openDB();
        TestUtil.createTempTable(con, "testbytea", "val bytea, id int");
        con.setAutoCommit(true);
    }
    private void prepareTestBytesFromDriverClass() throws Exception {
        File file = new File("jars", "postgresql.jar");
        int size = (int) file.length();
        test_bytes = new byte[size];
        FileInputStream fis = new FileInputStream(file);
        fis.read(test_bytes);
        fis.close();
    }
    
    public void testReadAndCompare() throws Exception {
        PreparedStatement ps = con.prepareStatement("INSERT INTO testbytea VALUES(?, 1)");
        ps.setBytes(1, test_bytes);
        assertEquals(1, ps.executeUpdate());
        ResultSet rs = con.createStatement().executeQuery("SELECT * from testbytea");
        assertTrue(rs.next());
        InputStream is = rs.getBinaryStream(1);
        int read = 0;
        int v;
        while ((v = is.read()) != -1) {
            assertEquals("Wrong byte value for position " + read, 
                    test_bytes[read], (byte) v);
            read++;
        }
        assertEquals("Wrong size of storead and readed bytes", test_bytes.length, read);
    }
    
     public void testReadAndCompareAndTrim() throws Exception {
        PreparedStatement ps = con.prepareStatement("INSERT INTO testbytea VALUES(?, 1)");
        ps.setBytes(1, test_bytes);
        assertEquals(1, ps.executeUpdate());
        Statement s = con.createStatement();
        
        int fieldSize = 1000;
        s.setMaxFieldSize(fieldSize);
        ResultSet rs = s.executeQuery("SELECT * from testbytea");
        
        assertTrue(rs.next());
        InputStream is = rs.getBinaryStream(1);
        int read = 0;
        int v;
        while ((v = is.read()) != -1) {
            assertEquals("Wrong byte value for position " + read, 
                    test_bytes[read], (byte) v);
            read++;
        }
        assertEquals("Wrong size of storead and readed bytes", fieldSize, read);
    }
}
