/*
 * $URL$
 * $Id$
 * $Revision$
 */

package org.postgresql.util.dataio;

import java.io.IOException;
import java.io.InputStream;

/**
 *
 * @author Radosław Smogura
 */
public class PGByteaTextInputStream_9_0_1 extends InputStream {
    private final long trimSize;
    
    private int position;
    
    /** Holds number of read bytes, to trim stream if needed. */
    private volatile long read;
    
    private final byte[] byteaBytes;

    public PGByteaTextInputStream_9_0_1(byte[] byteaBytes) {
        this(byteaBytes, Long.MAX_VALUE);
    }
    
    public PGByteaTextInputStream_9_0_1(byte[] byteaBytes, long trimSize) {
        this.trimSize = trimSize;
        this.byteaBytes = byteaBytes;
        position = 2;
    }

    @Override
    public int read() throws IOException {
        final byte[] bytes = byteaBytes; //Avoid getfield opcode
        if (!(position < byteaBytes.length))
            return -1;
        
        if (read++ == trimSize)
            return -1;
        
        byte b1 = gethex(bytes[position++]);
        byte b2 = gethex(bytes[position++]);
        
        return ((b1 << 4) | b2);
    }
    
    /** Copy from PGBytea */
    private byte gethex(byte b) {
        // 0-9 == 48-57
        if (b <= 57)
            return (byte)(b - 48);

        // a-f == 97-102
        if (b >= 97)
            return (byte)(b - 97 + 10);

        // A-F == 65-70
        return (byte)(b - 65 + 10);
    }
}
